/*
 * Copyright (c) 1998 Peter Bentley (pete@sorted.org)
 * 
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice and this permission notice
 * appear in all copies of the software and related documentation
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL PETER BENTLEY BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
 * ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 * 
 * $Id: wtype.h,v 1.3 1998/05/19 21:08:17 pete Exp $
 * 
 */

#ifndef WTYPE_H
#define WTYPE_H

typedef struct _WidgetType WidgetType;


typedef GtkWidget	*(*ToWidgetFunc)( WidgetType *w, proplist_t value );
typedef proplist_t	(*FromWidgetFunc)( WidgetType *w, GtkWidget * );
typedef int		(*TypePropFunc)( WidgetType *w, proplist_t value );

#define VALUE_TO_WIDGET( type, value )     (type->to_widget)(type, value)
#define VALUE_FROM_WIDGET( type, widget )  (type->from_widget)(type, widget)
#define TYPE_MATCH( type, value)           (type->match)(type, value)

struct _WidgetType {
    char		*name;
    int			priority;
    ToWidgetFunc	to_widget;
    FromWidgetFunc	from_widget;
    TypePropFunc	match;
    TypePropFunc	derive;
    void		*aux;		/* Extra data for this type */
    proplist_t		pname;		/* Name converted to a proplist */
};


WidgetType	*WidgetTypeByName( const char *name );
WidgetType	*WidgetTypeInfer( proplist_t value );
int		WidgetTypeDerive( char *base, char *name, proplist_t data );


#endif /* WTYPE_H */
